#!/bin/bash

easy_rsa=/var/lib/easy-rsa/easyrsa3/easyrsa
pki=/etc/pki_db

function init(){
  rm -rf $pki
  $easy_rsa init-pki > /dev/null 2>&1
  $easy_rsa --batch build-ca nopass > /dev/null 2>&1
}

function add(){
  $easy_rsa build-server-full $1 nopass > /dev/null 2>&1
  cp -f $pki/issued/$1.crt /etc/pki/tls/certs/server.crt
  cp -f $pki/private/$1.key /etc/pki/tls/private/server.key
  cp -f /etc/httpd/conf.d/ssl.conf.custom /etc/httpd/conf.d/ssl.conf
}

function revoke(){
  $easy_rsa --batch revoke $1 > /dev/null 2>&1
}

function show_ca_cert(){
  if [ -f $pki/ca.crt ];then
    cat $pki/ca.crt
  fi
}

function fqdn_check(){
  if [[ "$1" =~ ^[0-9a-z.-]*$ ]];then
    return 0
  fi
  return 1
}

function clean(){
  rm -rf $pki
  rm -f /etc/pki/tls/certs/server.crt
  rm -f /etc/pki/tls/private/server.key
  cp -f /etc/httpd/conf.d/ssl.conf.default /etc/httpd/conf.d/ssl.conf
}

function usage(){
  echo -e "usage: $0"
  echo -e "  -init \t\tinitialize CA database."
  echo -e "  -show_ca \t\tshow CA certificate."
  echo -e "  -add <hostname> \tadd hostname server certificate."
  echo -e "  -revoke <hostname> \trevoke hostname server certificate."
  echo -e "  -clean \t\tremove all certificates, keys."
  exit 1
}

case $1 in
  "-init")
    init
  ;;
  "-add")
    if [ -z "$2" ];then
      usage
    else
      fqdn_check $2
      if [ "$?" == "0" ];then
        add $2
      else
        echo "invalid hostname."
        exit 1
      fi
    fi
  ;;
  "-revoke")
    if [ -z "$2" ];then
      usage
    else
      fqdn_check $2
      if [ "$?" == "0" ];then
        revoke $2
      else
        echo "invalid hostname."
        exit 1
      fi
    fi
  ;;
  "-clean")
    clean
  ;;
  "-show_ca")
    show_ca_cert
  ;;
  *)
    usage
  ;;
esac

exit 0
