#!/opt/rh/rh-ruby23/root/usr/bin/ruby
# -*- coding: utf-8 -*-

require 'rubygems'
require 'open3'
require 'yaml'
require 'fileutils'
require 'active_record'

class DatabaseManager

  BASE_PATH = '/var/lib/redmine'.freeze
  CONFIG_PATH = (BASE_PATH + '/config/database.yml').freeze
  CAT_CMD = '/bin/cat'.freeze
  SED_CMD = '/usr/bin/sed'.freeze
  PSQL_CMD = '/usr/bin/psql'.freeze

  VMBASE_PATH = (BASE_PATH + '/plugins/sidfm-vm-server').freeze
  VMINIT_FILE = (VMBASE_PATH + '/ROR/db/vm_init.psql').freeze

  PASSENGER_CONFIG = '/opt/rh/rh-ruby23/root/usr/local/bin/passenger-config'.freeze

  private

  def initialize(argc = 'db_manage', argv = [])
    @argc = argc.downcase.to_sym
    @argv = argv
    run
  end

  def usage
    STDERR.puts "usage(show):"
    STDERR.puts " #{@argc} show"
    STDERR.puts "usage(config):"
    STDERR.puts " #{@argc} config <dbname> <dbhost> <dbport> <username> <password>"
    STDERR.puts "usage(test):"
    STDERR.puts " #{@argc} test"
    STDERR.puts "usage(init):"
    STDERR.puts " #{@argc} init"
    exit 1
  end

  def get_config
    config = YAML.load(File.read(CONFIG_PATH))
    ret = nil
    if config.has_key?('production') then
      ret = config['production']
    end
    unless ret then
      STDERR.puts 'invalid config'
      exit 1
    end
    ret
  end

  def run
    usage if @argv.size == 0
    case @argv[0].downcase.to_sym
    when :config then run_config
    when :show then run_show
    when :init then run_init
    when :test then run_test
    else usage
    end
  end

  def run_show
    usage if @argv.size != 1
    config_production = get_config
    puts "Database Name: #{config_production['database']}"
    puts "Database Host: #{config_production['host']}"
    puts "Database Port: #{config_production['port'] ? config_production['port'].to_i : 5432}"
    puts "Database Username: #{config_production['username']}"
    puts "Database Password: #{config_production['password'] ? config_production['password'][0, 2] + '************' : (none)}"
    exit 0
  end

  def run_config
    usage if @argv.size != 6
    begin
      FileUtils.cp(CONFIG_PATH, CONFIG_PATH + '.old')
      config = {
        'production' => {
          'adapter' => 'postgresql',
          'database' => @argv[1],
          'host' => @argv[2],
          'port' => @argv[3].to_i,
          'username' => @argv[4],
          'password' => @argv[5],
          'encoding' => 'utf8',
        },
      }
      File.open(CONFIG_PATH, 'w') do |f|
        f.write(YAML.dump(config))
      end
      pid = Process.spawn(PASSENGER_CONFIG, 'restart-app', BASE_PATH, :in => '/dev/null', :out => '/dev/null', :err => '/dev/null')
      th = Process.detach(pid)
      th.join(60)
      STDERR.puts "success"
    rescue Exception => e
      FileUtils.cp(CONFIG_PATH + '.old', CONFIG_PATH)
      STDERR.puts "failure"
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    end
    exit 0
  end

  def run_init
    usage if @argv.size != 1
    config_production = get_config
    begin
      ENV['PGPASSWORD'] = config_production['password']
      result = Open3.pipeline("#{CAT_CMD} #{VMINIT_FILE}", "#{SED_CMD} -e 's/OWNER TO sidfmvmadm/OWNER TO #{config_production['username']}/'", "#{PSQL_CMD} -h #{config_production['host']} -p #{config_production['port'] ? config_production['port'] : 5432} -U #{config_production['username']} #{config_production['database']}")
      result_cat_status = result[0].exitstatus
      result_psql_status = result[1].exitstatus
      if result_cat_status == 0 and result_psql_status == 0 then
        STDERR.puts 'success'
      else
        STDERR.puts 'failure'
        exit 1
      end
    rescue Exception => e
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    ensure
      ENV['PGPASSWORD'] = nil
    end
    exit 0
  end

  def run_test
    usage if @argv.size != 1
    config_production = get_config
    begin
      ActiveRecord::Base.establish_connection(config_production)
      eval 'class IssueStatus < ActiveRecord::Base; end'
      is_all = IssueStatus.all
      if IssueStatus.count != 4 then
        STDERR.puts 'invalid database'
        exit 1
      end
      is_all_ids = is_all.collect {|x| x.id }
      unless (is_all_ids - [ 1, 2, 3, 4]).size == 0 and
          ([ 1, 2, 3, 4 ] - is_all_ids).size == 0 then
        STDERR.puts 'invalid database: maybe reuse database'
        exit 1
      end
      puts 'success'
    rescue PG::ConnectionBad => e
      STDERR.puts 'Database Connection/Access Failed:'
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    rescue ActiveRecord::StatementInvalid => e
      if e.cause.is_a?(PG::UndefinedTable) then
        STDERR.puts 'database not initialized'
        exit 1
      else
        STDERR.puts 'Unknown Exception:'
        STDERR.puts e.message
        if e.backtrace then
          e.backtrace.each do |i|
            STDERR.puts i
          end
        end
      end
      exit 1
    rescue Exception => e
      STDERR.puts 'Unknown Exception:'
      STDERR.puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          STDERR.puts i
        end
      end
      exit 1
    end
    exit 0
  end

end

if $0 == __FILE__ then
  id = DatabaseManager.new($0, ARGV)
end
