#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class UpdateShortDescription

  LOG_FILE = '/var/lib/redmine/log/update_sdesc.log'.freeze

  private

  def initialize
    run
  end

  def double_puts(*a)
    File.open(LOG_FILE, 'a') do |f|
      if a.respond_to?(:each) then
        a.each do |i|
          f.puts i
          STDERR.puts i
        end
      else
        f.puts a
        STDERR.puts a
      end
    end
  end

  public

  def run
    begin

      double_puts 'update short description'

      double_puts '*** step1 (find no short-description issues) ***'
      htmlids = SidfmdIssue
        .where('sidfmd_issues.html_id > 0')
        .where('sidfmd_issues.short_description IS NULL')
        .select(:html_id).distinct(:html_id).collect {|x| x.html_id }
      double_puts " htmlids = #{htmlids.join(',')}"

      if htmlids.size > 0 then
        double_puts "*** step 2 (process issues) ***"
        api = Sidfm.sidapi
        if api then
          # 100レコード毎に処理。
          cnt = 1
          max_cnt = (htmlids.size.to_f / 100.0).ceil
          htmlids.each_slice(100) do |targets|
            double_puts " step #{cnt}/#{max_cnt}"
            cnt += 1
            double_puts " processing htmlid count = #{targets.size}"
            sdescs = api.vuln_short_descriptions(targets)
            if sdescs.respond_to?(:'[]') and sdescs['vulns'] and sdescs['vulns'].respond_to?(:size) then
              sdescs = sdescs['vulns']
              double_puts " api result count = #{sdescs.size}"
              sdescs.each do |sdesc|
                html_id = (sdesc['id'] || '0').to_i
                short_description = sdesc['short_description'] || ''
                if html_id > 0 and short_description then
                  SidfmdIssue
                    .where(:html_id => html_id)
                    .where('sidfmd_issues.short_description IS NULL OR LENGTH(sidfmd_issues.short_description) = 0')
                    .update_all(:short_description => short_description)
                else
                  double_puts " invalid api data: #{sdesc.inspect}"
                end
              end
            else
              double_puts " invalid api response: #{sdescs.inspect}"
            end
          end
        else
          double_puts " invalid api"
        end
      else
        double_puts " no updatable issues."
      end


      double_puts '*** FINISHED ***'

    rescue ActiveRecord::StatementInvalid => e
      if e.cause.is_a?(PG::UndefinedTable) then
        double_puts 'database not initialized'
        exit 1
      else
        double_puts 'Unknown Exception:'
        double_puts e.message
        if e.backtrace then
          e.backtrace.each do |i|
            double_puts i
          end
        end
      end
      exit 1
    rescue SystemExit => e
      double_puts "exit"
      exit 1
    rescue Exception => e
      double_puts 'Unknown Exception:'
      double_puts e.message
      if e.backtrace then
        e.backtrace.each do |i|
          double_puts i
        end
      end
      exit 1
    end
    exit 0
  end

end
