# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmOpenidConnectSettingController < ApplicationController
  unloadable

  layout 'sidfm'
  self.main_menu = false
  protect_from_forgery with: :exception
  helper :sidfm

  private

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :create_common_data
  before_action :get_sidfm_roles
  before_action :check_roles
  before_action :get_vm_version
  before_action :clear_import_session, only: [ :index ]
  before_action :clear_export_session, only: [ :index ]

  def check_roles
    if !@sidfm_roles.include?(:admin) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def get_selectable_sidfm_roles
    get_sidfm_project
    roles = Role.where(:name => [ ROLE_SIDFM_MNG, ROLE_SIDFM_USE, ROLE_SIDFM_USR ])
  end
  
  public
  
  def index
    @settings = (Setting.send('plugin_redmine_openid_connect')).symbolize_keys
    @settings[:enabled] = (@settings[:enabled] ? true : false) #enabledがある時は、文字列のfalseになっている
    @settings[:dynamic_config_expiry] = 1 if !@settings[:dynamic_config_expiry] or @settings[:dynamic_config_expiry].to_s.size == 0
    role_id = Sidfm.openid_connect_settings_user_role_id
    role = (role_id.length > 0 ? Role.find_by(:id => role_id) : nil)
    category_ids = Sidfm.openid_connect_settings_user_category_ids
    @settings[:sidfm_user_role] = (role ? { :id => role.id, :name => role.name } : nil)
    #sidfm_categoriesは今のところ1つのみ設定可能になっている
    @settings[:sidfm_categories] = (category_ids.length > 0 ? SidfmdCategory.where(:id => category_ids.split(',')).map{|c| { :id => c.id, :name => c.name, :desc => c.desc }} : nil)
    @roles = get_selectable_sidfm_roles.order(%|
      CASE name 
        WHEN 'SIDfmVM報告者' THEN 0
        WHEN 'SIDfmVM利用者' THEN 1
        WHEN 'SIDfmVM閲覧者' THEN 2
      END
    |)
    @custom_categories = SidfmdCategory.where(:is_system => false)
                                       .order('LOWER(name COLLATE "C")')
                                       .select(:id, :name, :desc)
  end

  def modify
    f_params = params.fetch(:form, {}).permit(:enabled, :client_id, :openid_connect_server_url , :client_secret, :scopes, :disable_ssl_validation, :user_role_id, :user_category_id)
    if (f_params[:enabled] != 'enable' and f_params[:enabled] != 'disable') or 
       (f_params[:disable_ssl_validation] != 'enable' and f_params[:disable_ssl_validation] != 'disable') or
       !f_params[:user_role_id] or !f_params[:user_category_id] #user_category_idsは今のところ1つのみ設定可能
    then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
 
=begin  
    expire_days = f_params[:expire_days]
    if expire_days.size > 0 then
      if (expire_days =~ /\A[0-9]+\z/).nil? then
        redirect_to :action => :index, :notice => '不正な操作です。' and return
      end
      expire_days = expire_days.to_i.to_s 
    end
=end

    user_role_id = f_params[:user_role_id]
    if (user_role_id =~ /\A[0-9]+\z/).nil? then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    selectable_roles = get_selectable_sidfm_roles
    if !selectable_roles.where(:id => user_role_id).exists? then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end

    user_category_id = f_params[:user_category_id]
    if (user_category_id =~ /\A[0-9]+\z/).nil? then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end 
    if SidfmdCategory.where(:id => user_category_id).where(:is_system => true).exists? then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
   
=begin
    begin
      user_category_ids = JSON.parse(f_params[:user_category_ids] || '[]')
    rescue JSON::ParserError => e
      log_exception(e)
      redirect_to :index, :notice => '不正な操作です。' and return
    end

    if user_category_ids.any? then
      temp_cats = SidfmdCategory.where(:id => user_category_ids)
      if temp_cats.where(:is_system => true).exists? then
        redirect_to :index, :notice => '不正な操作です。' and return
      end
      user_category_ids = temp_cats.pluck(:id)
    end

    if f_params[:enabled] == 'enable' then
      if user_role_id.size == 0 or user_category_ids.size == 0 then
        redirect_to :index, :notice => '設定の変更に失敗しました。' and return
      end
    end
=end
    #項目は、pluginの設定値に合わせる
    settings = {
      :client_id => f_params[:client_id],
      :openid_connect_server_url => f_params[:openid_connect_server_url],
      :client_secret => f_params[:client_secret],
      :scopes => f_params[:scopes],
      :dynamic_config_expiry => '',
      :group => '',
      :admin_group => '',
      :login_selector => 'false',
      :create_user_if_not_exists => 'true'
    }
    settings[:enabled] = 'false' if f_params[:enabled] == 'enable'
    settings[:disable_ssl_validation] = 'true' if f_params[:disable_ssl_validation] == 'enable'
   
    begin
      Sidfm.transaction do
        Setting.send 'plugin_redmine_openid_connect=', settings.stringify_keys
        Sidfm.openid_connect_settings_user_role_id = user_role_id
        Sidfm.openid_connect_settings_user_category_ids = user_category_id 
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :index, :notice => '設定の変更に失敗しました。' and return
    end

    redirect_to :action => :index 
  end
 
end
