# -*- coding: utf-8 -*-
require 'sidfm_controller_support'

class SidfmSystemSettingController < ApplicationController

  unloadable

  layout 'sidfm'
 
  self.main_menu = false

  helper :sidfm
  protect_from_forgery with: :exception

  include SidfmControllerSupport
  before_action :update_session_updated_at
  before_action :create_common_data
  before_action :get_sidfm_roles
  before_action :check_roles
 
  before_action :get_vm_version
  before_action :clear_import_session
  before_action :clear_export_session

  private

  def check_roles
    unless @sidfm_user then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
    if !@sidfm_roles.include?(:admin) then
      redirect_to home_path, :notice => af_log_user('許可されていない操作です。') and return
    end
  end

  def get_password_policy_settings
    password_policy_settings = {}
    password_policy_settings[:password_min_length] = Setting.password_min_length.to_i
    password_policy_settings[:available_char_types] = Sidfm.get_password_available_char_types.split('/')
    password_policy_settings[:available_symbols] = Sidfm.get_password_available_symbols.split('')
    password_policy_settings[:is_chars_types_mix_needed] = Sidfm.get_is_password_char_type_mix_needed
    password_policy_settings[:is_permited_password_as_same_username] = Sidfm.get_is_permited_password_as_same_username
    password_policy_settings[:is_permited_password_includes_username] = Sidfm.get_is_permited_password_includes_username
    return password_policy_settings
  end

  #def escape_special_chars(text)
  #  text = text.gsub(/&/,'&amp;')
  #  text = text.gsub(/<|>|"|'/, '<' => '&lt;', '>' => '&gt;', '"' => '&quot;', "'" => '&#39;', '`' => '&#x60;')
  #end

  public 
  
  def index
    @password_policy_settings = get_password_policy_settings
    all_available_char_types = Sidfm::AVAILABLE_PASSWORD_CHAR_TYPES.stringify_keys
    @available_char_types = @password_policy_settings[:available_char_types].map{|c_type| all_available_char_types[c_type]}
  end

  def modify_url
    form_params = params.fetch(:form, {}).permit(:url, :protocol)
    url = form_params[:url]
    protocol = form_params[:protocol]
    if !url || url == "" then
      redirect_to :action => :index, :notice => 'ホスト名とパスが入力されていません。' and return
    end
    if !protocol || (protocol != 'http' && protocol != 'https') then
      redirect_to :action => :index, :notice => 'プロトコルが不正です。' and return
    end  
    begin 
      Setting.transaction do
        Setting.protocol = protocol
        Setting.host_name = url
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'URLの変更に失敗しました。' and return
    end
    Rails.cache.clear
    redirect_to :action => :index
  end
  
  def modify_mail_address
    form_params = params.fetch(:form, {}).permit(:mail_address)
    mail_address = form_params[:mail_address]
    if !mail_address.match(/\A([^@\s]+)@((?:[-a-z0-9]+\.)+[a-z]{2,})\z/i) then
      redirect_to :action => :index, :notice => 'メールアドレスが不正です。' and return
    end
    begin
      Setting.transaction do
        Setting.mail_from = mail_address
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'メールアドレスの変更に失敗しました。' and return
    end
    Rails.cache.clear
    redirect_to :action => :index
  end

  def modify_password_policy
    f_params = params.fetch(:form, {}).permit(:min_password_length, :available_chars_types, :available_symbols, :is_chars_types_mix_needed, :permit_password_as_same_username, :permit_includes_username)
    min_password_length = f_params[:min_password_length]
    is_chars_types_mix_needed = f_params[:is_chars_types_mix_needed]
    is_petmited_same_username = f_params[:permit_password_as_same_username]
    is_petmited_includes_username = f_params[:permit_includes_username]
    begin 
      available_char_types = JSON.parse(f_params[:available_chars_types])
      available_symbols = JSON.parse(f_params[:available_symbols])
    rescue JSON::ParserError => e
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    if !min_password_length || !min_password_length.match(/[0-9]/) || !is_petmited_same_username || !is_petmited_includes_username then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    #最小文字数が不正でないかどうかチェック
    default_min_length = 8
    max_length = 16
    if min_password_length.to_i < default_min_length then
      redirect_to :action => :index, :notice => '設定可能な最小文字数は' + default_min_length.to_s + 'です。' and return
    elsif min_password_length.to_i > max_length then
      redirect_to :action => :index, :notice => '設定可能な最小文字数の最大値は' + max_length.to_s + 'です。' and return
    end

    begin
      Setting.transaction do
        Setting.transaction do
          Setting.password_min_length = min_password_length
        end
        Sidfm.transaction do
          Sidfm.set_password_available_char_types available_char_types.join('/')
          Sidfm.set_password_available_symbols available_symbols.join('')
          Sidfm.set_is_password_char_type_mix_needed is_chars_types_mix_needed if is_chars_types_mix_needed 
          Sidfm.set_is_permited_password_as_same_username is_petmited_same_username
          Sidfm.set_is_permited_password_includes_username is_petmited_includes_username
        end
      end
    rescue Exception => e
      logger.info(e)
      redirect_to :action => :index, :notice => 'パスワードポリシーの変更に失敗しました。' and return
    end
    Rails.cache.clear
    redirect_to :action => :index
  end

  def modify_sync_hosts_num
    f_params = params.fetch(:form, {}).permit(:hosts_num)
    hosts_num = f_params[:hosts_num]
    if !hosts_num then
      redirect_to :action => :index, :notice => '不正な操作です' and return
    end
    hosts_num = hosts_num.to_i
    if hosts_num < 1 || hosts_num > Sidfm::MAX_PARALLEL_HOSTS then
      redirect_to :action => :index, :notice => '不正な操作です' and return
    end
    begin 
      Sidfm.parallel_hosts = hosts_num
    rescue Exception => e
      logger.info(e)
      redirect_to :action => :index, :notice => '更新に失敗しました' and return
    end
    redirect_to :action => :index
  end

  def set_vulsync_interval
    f_params = params.fetch(:form, {}).permit(:interval)
    interval = f_params[:interval]
    unless interval then
      redirect_to :action => :index, :notice => '不正な操作です' and return
    end
    interval = interval.to_i
    if interval < Sidfm::MIN_VULSYNC_INTERVAL or interval > Sidfm::MAX_VULSYNC_INTERVAL then
      redirect_to :action => :index, :notice => '不正な操作です' and return
    end
    begin
      Sidfm.vulsync_interval = interval
    rescue Exception => e
      logger.info(e)
      redirect_to :action => :index, :notice => '更新に失敗しました' and return
    end
    redirect_to :action => :index
  end

  def set_sesstion_timeout
    f_params = params.fetch(:form, {}).permit(:minutes)
    minutes = f_params[:minutes]
    if !minutes || minutes.size == 0 then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end
    begin
      minutes = minutes.to_i
      Sidfm.transaction do
        Sidfm.session_timeout = minutes
      end
    rescue Exception => e
      logger.info(e)
      redirect_to :action => :index, :notice => 'セッションタイムアウト時間の変更に失敗しました。' and return
    end
   
    redirect_to :action => :index
  end

  def set_max_rules
    f_params = params.fetch(:form, {}).permit(:max_rules)
    max_rules = f_params[:max_rules]
    unless max_rules then
      redirect_to :action => :index, :notice => '不正な操作です。' and return
    end

    begin 
      max_rules = max_rules.to_i
      if max_rules > SidfmdRule::MAXIMUM_LIMIT || max_rules < 0 then
        redirect_to :action => :index, :notice => '不正な操作です。' and return
      end
      cnt_rules = SidfmdRule.count
      if max_rules < cnt_rules then
        redirect_to :action => :index, :notice => "登録済みのルール件数(#{cnt_rules})を下回っているため、ルールの最大登録件数は変更されませんでした。" and return
      end 
      Sidfm.max_rules = max_rules
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'ルールの最大登録件数の変更に失敗しました。' and return
    end
  
    redirect_to :action => :index
  end

  def set_restrict_mail_notification_setting
    f_params = params.fetch(:form, {}).permit(:restrict_mail_notification_setting)
    restrict_mail_notification_setting = f_params[:restrict_mail_notification_setting]
    Sidfm.restrict_mail_notification_setting = ('true' == restrict_mail_notification_setting)
    redirect_to :action => :index
  end

  def set_syncing_check_span
    f_params = params.fetch(:form, {}).permit(:syncing_check_span)
    syncing_check_span = f_params[:syncing_check_span].to_i
    unless syncing_check_span >=2 then
      redirect_to :action => :index, :notice => '2日より短い値は設定できません。' and return
     end
    Sidfm.syncing_check_span = syncing_check_span
    redirect_to :action => :index
  end

  def set_max_history_download_size
    f_params = params.fetch(:form, {}).permit(:max_history_download_size)
    begin
      size = f_params[:max_history_download_size].to_i
      if size > Sidfm::HISTORY_MAX_DOWNLOAD_SIZE or size < Sidfm::HISTORY_MIN_DOWNLOAD_SIZE then
        redirect_to :action => :index, :notice => '不正な操作です。' and return
      end
      Sidfm.set_history_max_download_size = size
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => '履歴画面の最大ダウンロード件数の変更に失敗しました。' and return
    end 

    redirect_to :action => :index
  end

  def set_password_period
    f_params = params.fetch(:form, {}).permit(:period, :is_disable_period)
    begin
      if f_params[:is_disable_period] then
        Setting.password_max_age = 0
      else
        period = f_params[:period].to_i
        if period < 1 or period > Sidfm::MAX_PASSWORD_PERIOD then
          redirect_to :action => :index, :notice => '不正な操作です。' and return
        end
        Setting.password_max_age = period
      end
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => 'パスワードの有効期限の変更に失敗しました。' and return
    end 

    redirect_to :action => :index
  end

  def set_vmdaemon_affect_product_status
    f_params = params.fetch(:form, {}).permit(:is_enable)
    is_enable = f_params[:is_enable].to_s == 'enable'
    begin
      Sidfm.vmdaemon_affect_product = is_enable
    rescue Exception => e
      log_exception(e)
      redirect_to :action => :index, :notice => '有効/無効の変更に失敗しました。' and return
    end
    redirect_to :action => :index
  end

end
