#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

module SidfmHistoriesHelper
  def history_status_name(status_id)
    case status_id.to_i 
      when SidfmdIssue::STATUS_NOTCONFIRMED then '未対策'
      when SidfmdIssue::STATUS_ONGOING then '対策中'
      when SidfmdIssue::STATUS_FIXED then '対策済'
      when SidfmdIssue::STATUS_NOTAFFECT then '影響無'
    end
  end

  def history_status_class_name(status_id)
    case status_id.to_i
      when SidfmdIssue::STATUS_NOTCONFIRMED then 'not-confirmed'
      when SidfmdIssue::STATUS_ONGOING then 'ongoing'
      when SidfmdIssue::STATUS_FIXED then 'fixed'
      when SidfmdIssue::STATUS_NOTAFFECT then 'notaffect'
    end
  end

  # サマリー表のチェックボックス
  def create_summary_history_chkbox(journal_id, checked)
    content = check_box_tag(
      'table[selected]',
      journal_id,
      checked.include?(journal_id),
      :class => 'historySelect',
    )
  end

  # サマリー表の日時
  def create_summary_history_created_on(created_on)
    created_on ? created_on.strftime('%Y/%m/%d %H:%M:%S') : "---"
  end

  # サマリー表のユーザ
  def create_summary_history_user(login_name, user_fullname, email_address, is_anonymous)
    user_info = is_anonymous ?
      "" : 
      "ログイン名: #{login_name}<br>氏名: #{user_fullname}<br>メールアドレス: #{email_address}"
    content = content_tag(
      :div,
      :data => { :toggle => "tooltip", :title => user_info, :html => "true", :placement => "right" }
    ) do
      concat(
        if is_anonymous then
          content_tag(
            :div,
            "削除済みユーザ",
            :class => "login-content text-danger"
          )
        else
          content_tag(
            :div,
            "#{login_name}",
            :class => "login-content"
          )
          .concat(
            content_tag(
              :div,
              :class => "email-content" 
            ) do
              concat(
                content_tag(:i, nil, :class => "far fa-envelope mr-1")
              )
              .concat(
                link_to(email_address, "mailto:#{email_address}")
              )
            end
          )
        end
      )
    end
  end

  # サマリー表の操作
  def create_summary_history_operation(operation, old_value, current_value)
    content = content_tag(
      :div,
      :class => "operation_detail_content"
    ) do
      concat(
        if operation then
          content_tag(
            :div,
            :class => "operation_detail_title text-left font-weight-bold"
          ) do
            concat( content_tag(:span, "#{operation}") )
          end
        else
          ""
        end
      )
      .concat(
        if operation == 'ステータス変更' and old_value and current_value then
          content_tag(
            :div,
            :class => "text-left"
          ) do
            concat(
              content_tag(
                :span,
                "#{history_status_name(old_value)}",
                :class => "status-btn btn_st_#{history_status_class_name(old_value)} mr-1"
              )
            )
            .concat("から")
            .concat(
              content_tag(
                :span,
                "#{history_status_name(current_value)}",
                :class => "status-btn btn_st_#{history_status_class_name(current_value)} ml-1 mr-1"
              )
            )
            .concat("へ")
          end
        elsif operation == '対策予定日の変更' then
          content_tag(  
            :div,
            :class => "text-left"
          ) do
            concat(
              content_tag(:span, "#{old_value ? old_value : "未設定"}", :class => "font-weight-bold mr-1")
            )
            .concat("から")
            .concat(
              content_tag(:span, "#{current_value ? current_value : "未設定"}", :class => "font-weight-bold mr-1 ml-1") 
            )
            .concat("へ")
          end
        else
          "---"
        end
      )
    end
  end

  #サマリー表のコメント
  def create_summary_history_comment(comment, is_deleted_comment, journal_id, sidfm_roles)
    content = content_tag(
      :div,
      :class => "comment-content d-flex"
    ) do
      concat(
        content_tag(
          :div,
          :class => "deleted-comment-font #{sidfm_roles.include?(:use) ? 'w-80' : 'w-100'}",
          :data => {
            :toggle => "tooltip",
            :title => "変更しました!",
            :placement => "top",
            :trigger => "manual",
          },
          :id => "wrap-comment-#{journal_id}"
        ) do
          concat(
            content_tag(
              :div,
              :class => "summary-history-comment",
              :data => {
                :toggle => "tooltip",
                :title => "#{is_deleted_comment ? '' : comment}",
                :placement => "right",
              },
              :id => "comment-#{journal_id}"
            ) do
              concat(
                if is_deleted_comment then
                  content_tag(:span, comment, :class => "slanted-font")
                else
                  comment
                end
              )
            end
          )
        end
      )
      .concat(
        if sidfm_roles.include?(:use) and !sidfm_roles.include?(:host_limited_admin) then
          content_tag(:div, :class => "dropdown ml-auto") do
            concat(
              button_tag(
                "編集",
                :type => "button",
                :class => "dropdown-toggle btn btn-xs btn-info open-edit-comment-menu",
                :data => {
                  :toggle => "dropdown",
                  :filp => "false",
                  :boundary => "window",
                  :journalid => journal_id
                },
                'aria-haspopup' => 'true',
                'aria-expanded' => 'false'
              )
            )
            .concat(
              content_tag(
                :div,
                :class => "dropdown-menu edit-comment-menu",
                :data => { :menu => "edit-comment" }
              ) do
                concat(
                  content_tag(
                    :div,
                    :class => "dropdown-item-text p-0"
                  ) do
                    concat(
                      label_tag(
                        nil,
                        "コメント",
                        :class => "font-weight-bold mb-1",
                        :for => "edit-comment-textarea-#{journal_id}"
                      )
                    )
                    .concat(
                      hidden_field_tag(
                        nil,
                        (is_deleted_comment ? '' : comment),
                        :id => "original-comment-#{journal_id}"
                      )
                    )
                    .concat(
                      text_area_tag(
                        'form[comment]',
                        (is_deleted_comment ? '' : comment),
                        :id => "edit-comment-textarea-#{journal_id}",
                        :class => 'form-control edit-comment-textarea',
                        :rows => 5,
                        :cols => 25,
                        :data => { :journalid => journal_id, :editfield => 'comment' }
                      )
                    )
                    .concat(
                      content_tag(
                        :div,
                        :class => "d-flex justify-content-around mt-1"
                      ) do
                        concat(
                          button_tag(
                            '削除',
                            :type => 'button',
                            :class => 'btn btn-sm btn-danger delete-comment',
                            :data => { :journalid => journal_id },
                            :disabled => is_deleted_comment ? 'disabled' : false
                          )
                        )
                        .concat(
                          button_tag(
                            '変更',
                            :type => 'button',
                            :class => 'btn btn-sm btn-primary edit-comment',
                            :data => { :journalid => journal_id },
                            :disabled => 'disabled'
                          )
                        )
                      end
                    )
                  end
                )
              end
            )
          end
        end
      )
    end            
  end

  # サマリー表の脆弱性
  def create_summary_history_vul_title(vul_title, vul_short_description, host_id, html_id)
    content = content_tag(
      :div,
      :data => { :toggle => "tooltip", :title => vul_short_description.to_s, :boundary => "window" }
    ) do
      concat(
        link_to(
          {
            :controller => :sidfm_vulnerability,
            :action => :index,
            :id => html_id,
            :'form[history]' => [ host_id, html_id ].to_json
          }
        ) do
          concat(
            content_tag(
              :div,
              :class => "inline-block max-width-100 text-truncate",
              :title => vul_title
            ) do
              concat(
                content_tag(
                  :span,
                  "[#{html_id > 0 ? html_id : "KB#{-html_id}"}] #{vul_title}",
                  :class => "align-middle"
                )
              )
            end
          )
        end
      )
    end
  end

  # サマリー表のホスト名
  def create_summary_history_hostname(hostname, ext_name, display_name, host_id, html_id)
    contnet = content_tag(
      :div,
      :data => {
        :toggle => "tooltip",
        :title => "ホスト名: #{hostname}<br/>表示名: #{ext_name ? ext_name : "(未設定)"}",
        :placement => "right",
        :html => "true"
      }
    ) do
      concat(
        link_to(
          display_name,
          {
            :controller => :sidfm_host,
            :action => :index,
            :id => host_id,
            :'form[history]' => [ html_id, host_id ].to_json
          },
          :class => 'text-truncate g-text-ul g-letter-spacing-0_08'
        )
      )
    end
  end

  # サマリー表の対応状況
  def create_summary_history_status(status_id)
    content = content_tag(
      :span,
      "#{history_status_name(status_id)}",
      :class => "status-btn btn_st_#{history_status_class_name(status_id)}"
    )
  end

  # サマリー表の対策予定日
  def create_summary_history_due_date(due_date)
    content = due_date ? due_date.strftime('%Y/%m/%d') : "---"
  end

end



