#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

require 'digest/sha2'
require 'sidapi'

class SidfmdQueue < ActiveRecord::Base
  belongs_to :sidfmd_host

  has_one :sidfmd_queue_dist, :dependent => :destroy
  has_many :sidfmd_queue_pkgs, -> { order(:pkg) }, dependent: :destroy
  has_many :sidfmd_queue_apps, :dependent => :destroy

  has_many :sidfmd_queue_lapps, :dependent => :destroy

  has_many :sidfmd_queue_vuls, :dependent => :destroy

  has_one :sidfmd_queue_win, :dependent => :destroy

  # 有効なホストで、かつ、未処理のキュー
  scope :unproc_queues, -> { where(:done => false, :sidfmd_host => (SidfmdHost.where(:flg_delete => false))) }

  def update_digest
    dist = sidfmd_queue_dist
    str = '*'
    if dist then
      str << Digest::SHA2.hexdigest([ dist.ident, dist.version, dist.name ].join(','))
    end
    str << '**'
    pkgs = sidfmd_queue_pkgs.select(:pkg)
    if pkgs.size > 0 then
      str << Digest::SHA2.hexdigest(pkgs.collect {|x| x.pkg }.join(','))
    end
    str << '**'
    apps = sidfmd_queue_apps.select(:name, :version, :path)
    if apps.size > 0 then
      str << Digest::SHA2.hexdigest(apps.collect {|x| [ x.name, x.version, x.path ].join(',')}.join(','))
    end
    str << '**'
    lapps = sidfmd_queue_lapps.select(:product_id,:version_id)
    if lapps.size > 0 then
      str << Digest::SHA2.hexdigest(lapps.collect {|x| [ x.product_id, x.version_id ].join(',') }.join(','))
    end
    str << '**'
    win = sidfmd_queue_win
    if win then
      details = win.sidfmd_queue_win_details.select(:kb)
      str << Digest::SHA2.hexdigest(details.collect {|x| x[:kb] }.join(','))
    end
    str << '*'
    update_attribute :digest, str
  end

  # VM Agentで取得したアプリケーションとの照合
  def get_agent_products
    ret = []
    apps = self.sidfmd_queue_apps
    if apps then
      apps.each do |app|
        pv = app.get_agent_product
        if pv['product_id'] or pv['version_id'] then
          ret << pv
        end
      end
    end
    ret
  end

end
