/* coding: utf-8 */

//BootStrapTableのtdのスタイル。画面ごとに関数を作って設定する
function BootStrapTableCellStyle(value, row, index, field) {
    switch (field){
        case 'chkbox':
            return { classes: 'chk-box' };
        case 'priority':
            return { classes: 'priority' };
        case 'name':
            return { classes: 'name' };
        case 'category':
            return { classes: 'category' };
        case 'color':
            return { classes: 'color' };
        case 'edit':
            return { classes: 'edit' };
        default:
            return { classes: '' };
    }
}

$(function() {
    activate_sidebar_item('sidebar-item-ctags-manip');

    const summaryTableId = 'ctags-summary-table';
    const summaryTableElm = $('#' + summaryTableId);
    const pageName = 'ctags';
    const isUserAdmin = $('#is-user-admin').val() == 'true';
    const apiIssueRelatedCtagsPath = $('#sidfm_ctags_api_issue_related_ctags_path').val();

    //編集
    const enableEditCtag = () => {
        $('#' + summaryTableId + ' [data-edit="true"]').on('click', function(){
            const ctagData = $(this).data('ctag');
            setRegistModal({
                tagName: ctagData.name,
                priority: ctagData.priority,
                categoryName: ctagData.category_name,
                categoryId: ctagData.category_id,
                textColor: ctagData.color,
                bgColor: ctagData.bgcolor,
                actionName: 'edit',
                modifyId: ctagData.id
            });
            setRegistModalTitle('カスタムタグ情報の編集');
            $('#modalRegist').modal('show');
        });
    };

    //削除
    const enableDeleteCtag = () => {
        $('#' + summaryTableId + ' [data-delete="true"]').on('click', function(){
            (async () => {
                const deleteId = $(this).data('ctagid');
                const confMessage = `「${$(this).attr('data-name')}」を削除してもよろしいですか?`;
                const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                    additionalAlertMessage: '削除に失敗しました。'
                });
                if( roleOrCategoryError ) return;
                $('body').append(create_loading_window());
                const paramsData = { id: deleteId };
                const actionPath = $('#delete-ctag-path').val();
                $.ajax({
                    url: $('#sidfm_internal_api_check_related_rules_path').val(),
                    type: 'POST',
                    data: {'page_name': 'ctag', 'delete_id': deleteId, 'action_idx': 'delete'},
                }).done(function(data){
                    //console.log(data);
                    if( data['is_unchangeable_ctag'] ){
                        $('#checking-window').find('.progress-wait-area-label').text('');
                        setTimeout(function(){
                            alert(
                                'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、カスタムタグ削除することができません。\n' +
                                '※削除がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                            );
                            $('#checking-window').remove();
                        }, 100);
                        return false;
                    }
                    if(Object.keys(data['rule_messages']).length > 0){
                        $('#submit-related-rules-item-form').attr({
                            'data-params': JSON.stringify(paramsData),
                            'data-path': actionPath,
                            'data-action': 'delete',
                        });
                        $('#checking-window').remove();
                        display_related_rules(data);
                    }else{
                        let conf = confirm(confMessage);
                        if( !conf ) {
                            $('#checking-window').remove();
                            return;
                        }
                        dynamicSummaryTable.executeProcess({
                            paramsData,
                            path: actionPath,
                            additionalAlertMessage: '削除に失敗しました。',
                        });
                    }
                }).fail(function(){
                    alert('エラーが発生しました。');
                    $('#pages_form').submit();
                });;
            })();
        });
    };

    //サマリー表の動的処理のため(bootstrap-tableのため)にbindする。でないとイベントが効かない。
    const bindSummaryTableRowEvents = () => {
        enable_bootstrap_actions('#' + summaryTableId);
        enableEditCtag();
        enableDeleteCtag();
    };

    //サマリー表に合わせて、一括処理に使うデータを更新
    const modifyPageContent = (data) => {
        $('#all-ctag-names').val(JSON.stringify(data.all_names));
    };

    const dynamicSummaryTable = new DynamicSummaryTable({
        summaryTableId,
        bindSummaryTableRowEventsFunction: bindSummaryTableRowEvents,
        pageName,
        allFilteredIdsId: 'all-ctag-ids',
        chkboxSelector: `#${summaryTableId} .ctagSelect`,
        allChkboxSelector: `#${summaryTableId} .allCtagSelect`,
        bulkBtnSelector: '.action-btn',
        modifyPageContent
    });

    set_style_for_ActionArea_on_filter_dropdown_event();
    sticky_control('SummaryTableButtonsArea', pageName);
    $('#pages_form').on('submit', function(){ set_summary_table_lines(pageName, $('#pages_per_page').val()) });

    //検索
    const summaryTableFilter = new SummaryTableFilter({dynamicSummaryTable});
    summaryTableFilter.enableFilterChangeLink('.category-change'); //カテゴリ検索
    summaryTableFilter.enableFilterText(
       '#filter-name',
       '#reset-filter-name',
       { fieldSelector: '#filters-name-input' },
       'name'
    ); //ログイン名検索*/
    summaryTableFilter.enableResetAllFilters('#filter-clear'); //全リセット

    //登録/更新
    const SearchedCatsArea = $('#modalRegistSearchedCatsArea');
    const SearchedCatsInfo = { fetching: null };
    const selectedCtagCategoryIdElm = $('#selected-ctag-category-id');
    const ctagNameInputElm = $('#ctag-name-input');
    const ctagPriorityInputElm = $('#ctag-priority-input');

    //カテゴリ選択欄のリセット
    const resetSelectCategoryArea = () => {
        ctagNameInputElm.val('');
        selectedCtagCategoryIdElm.val('');
        $('#ctag-select-all-category').prop('checked', isUserAdmin);
        $('#ctag-select-category').prop('checked', !isUserAdmin);
        $('#ctag-category-input').val('').prop('disabled', isUserAdmin);
        SearchedCatsArea.hide();
    };

    //色選択欄のリセット
    const resetSelectColorArea = () => {
        $('#color-not-required').prop('checked', true);
        $('#color-required').prop('checked', false);
        $('#ctag-text-color-input').val('');
        $('#ctag-bg-color-input').val('');
        $('#select-color-area').addClass('disabled-item');
    };

    //新規登録/編集モーダルの初期化
    const setRegistModal = ({
        tagName,
        priority,
        categoryName,
        categoryId,
        textColor,
        bgColor,
        actionName,
        modifyId
    }) => {
        resetSelectCategoryArea();
        resetSelectColorArea();
        ctagNameInputElm.val(tagName);
        ctagPriorityInputElm.val(priority);
        $('#modify-ctag-category-id').val(categoryId);
        if( categoryId ) {
            $('#ctag-category-input').val(categoryName).prop('disabled', false);
            selectedCtagCategoryIdElm.val(categoryId);
            $('#ctag-select-all-category').prop('checked', false);
            $('#ctag-select-category').prop('checked', true);
        }
        if( textColor && textColor.length ) {
            $('#color-not-required').prop('checked', false);
            $('#color-required').prop('checked', true);
            $('#select-color-area').removeClass('disabled-item');
            $('#ctag-text-color-input').val(textColor);
            $('#ctag-bg-color-input').val(bgColor);
        }
        $('#modify-ctag-id').val(modifyId ? modifyId : '');
        $('#regist-action').val(actionName);
        $('#exec-regist').text(actionName === 'create' ? '登録' : '更新');
        toggleRegistButton();
    };

    const setRegistModalTitle = (title) => {
        $('#modalRegistTitle').text(title);
    };

    //新規登録
    $('#create-new-ctag-btn').on('click', function(){
        setRegistModal({
            tagName: '',
            priority: 0,
            categoryName: '',
            categoryId: '',
            textColor: '',
            bgColor: '',
            actionName: 'create'
        });
        setRegistModalTitle('カスタムタグの新規登録');
        $('#modalRegist').modal('show');
    });

    //カテゴリの「全体」か指定かの選択
    $('#modalRegist [name="ctag[is_category_all]"]').on('change', function(){
        $('#ctag-category-input').prop('disabled',  $('#ctag-select-all-category').prop('checked'));
    });

    //色の指定する/しないの選択  
    $('#modalRegist [name="ctag[is_color_required]"]').on('change', function(){
        const selectColorArea = $('#select-color-area');
        if( $('#color-required').prop('checked') ){
            selectColorArea.removeClass('disabled-item');
        }else{
            selectColorArea.addClass('disabled-item');
        }
    });

    //カテゴリ選択 
    $('#ctag-category-input').on('input', function(){
        const name = $(this).val();
        SearchedCatsArea.html('');
        selectedCtagCategoryIdElm.val('');
        if( SearchedCatsInfo.fetching ){
            SearchedCatsInfo.fetching.abort();
        }
        SearchedCatsInfo.fetching = $.post($('#api-search-category-path').val(), { name }, function(categories){
            const catElms = categories.map( cat => {
                return $('<div>').attr({
                    'data-catid': cat.id,
                    'data-catname': cat.name,
                    class: 'searched-cat-name'
                }).text(cat.name);
            });
            if( catElms.length ){
                $('#ctag-category-input-tooltip').tooltip('hide');
                SearchedCatsArea.show().append(catElms);
                SearchedCatsArea.find('[data-catname]').on('click', function(){
                    $('#ctag-category-input').val($(this).attr('data-catname'));
                    selectedCtagCategoryIdElm.val($(this).attr('data-catid'));
                    SearchedCatsArea.hide();
                });
            }else{
                $('#ctag-category-input-tooltip').tooltip(name.length ? 'show' : 'hide');
                SearchedCatsArea.hide();
            }
        });
    });

    //登録ボタンの活性/非活性
    function toggleRegistButton(){
        let isInvalid = !ctagNameInputElm.val().length || !ctagPriorityInputElm.val().length
        $('#exec-regist').prop('disabled', isInvalid);
    }

    ctagNameInputElm.on('input', function(){
        toggleRegistButton();
    });

    ctagPriorityInputElm.on('input', function(){
        toggleRegistButton();
    });

    //登録済みカスタムタグで、関連する脆弱性を取得
    const getIssueRelatedCtags = (ctagIds) => {
        return new Promise((resolve, reject) => {
            $.post(apiIssueRelatedCtagsPath, { ids: JSON.stringify(ctagIds) }, function(ctags){
                resolve(ctags.map(ctag => ctag.keyword));
            }).fail(function(){
                alert('エラーが発生しました。');
                $('#pages_form').submit();
                reject(false);
            });
        });
    };

    //登録済みカスタムタグで、カテゴリ変更時に関連するルールを取得
    const getCtagRelatedRules = (ctagIds, categoryId) => {
        return new Promise((resolve, reject) => {
            $('body').append(create_loading_window());
            const ctagCategory = {};
            ctagIds.forEach(ctagId => ctagCategory[ctagId] = categoryId);
            $.ajax({
                url: $('#sidfm_internal_api_check_related_rules_path').val(),
                type: 'POST',
                data: {'page_name': 'ctag', 'change_category_ctag_ids': JSON.stringify(ctagCategory), 'action_idx': 'category_change'},
            }).done(function(data){
                //console.log(data);
                if( data['is_unchangeable_ctag'] ){
                    $('#checking-window').find('.progress-wait-area-label').text('');
                    setTimeout(function(){
                        alert(
                            'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、カスタムタグのカテゴリを変更すすることができません。\n' +
                            '※変更がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                        );
                        $('#checking-window').remove();
                    }, 100);
                    resolve(false);
                }
                if(Object.keys(data['rule_messages']).length > 0){
                    $('#checking-window').remove();
                    display_related_rules(data);
                    resolve(false);
                }else{
                    resolve(true);
                }
            }).fail(function(){
                alert('エラーが発生しました。');
                $('#pages_form').submit();
                reject(false);
            });
        });
    }

    //登録実行
    $('#exec-regist').on('click', function(){
        const button = $(this);
        button.prop('disabled', true);
        const actionName = $('#regist-action').val();
        const modifyId = $('#modify-ctag-id').val();
        const alertMessage = (actionName === 'create' ? '登録' : '更新') + 'に失敗しました。';
        const isCategoryAll = $('#ctag-select-all-category').prop('checked');
        const priority = parseInt(ctagPriorityInputElm.val());
        const categoryId = (isCategoryAll ? '' : selectedCtagCategoryIdElm.val());
        const paramsData = {
            name: ctagNameInputElm.val(),
            priority,
            category_id: categoryId,
        };
        const actionPath = (actionName === 'create' ? $('#create-ctag-path').val() : $('#modify-ctag-path').val());
        if( !isCategoryAll && !categoryId ){
            alert(
                'カテゴリが設定されておりません。\n' +
                (isUserAdmin ? 'カテゴリを指定する場合は、' : '') +
                'カテゴリ名を検索後、表示されたカテゴリから選択して下さい。'
            );
            button.prop('disabled', false);
            return;
        }
        if( !priority && priority != 0 ){
            alert('優先度を数値で入力して下さい。');
            button.prop('disabled', false);
            return;
        }
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: alertMessage
            });
            if( roleOrCategoryError ) return;
            let issueRelatedCtags = [];
            if( actionName !== 'create' ){
                paramsData.id = modifyId;
                const prevCategoryId = String($('#modify-ctag-category-id').val());
                if( prevCategoryId !== String(categoryId) ){
                    $('#submit-related-rules-item-form').attr({
                        'data-params': JSON.stringify(paramsData),
                        'data-path': actionPath,
                        'data-action': 'modify-ctag'
                    });
                    //カテゴリが変わる場合は紐づく脆弱性がないかチェック
                    issueRelatedCtags = await getIssueRelatedCtags([modifyId]);
                    if( issueRelatedCtags === false ) {
                        $('#checking-window').remove();
                        button.prop('disabled', false);
                        return;
                    }
                    const IsNotRelatedRule = await getCtagRelatedRules([modifyId], categoryId);
                    if( IsNotRelatedRule === false ) {
                        $('#checking-window').remove();
                        button.prop('disabled', false);
                        return;
                    }
                }
            }
            $.post(
                $('#api-is-same-keyword-path').val(),
                { keyword: ctagNameInputElm.val(), modify_id: modifyId },
                function(data){
                    if( data.is_same_keyword ){
                        alert('既に登録時みのタグ名です。');
                        button.prop('disabled', false);
                        return;
                    }
                    if( $('#color-required').prop('checked') ){
                        paramsData.text_color = $('#ctag-text-color-input').val();
                        paramsData.bg_color = $('#ctag-bg-color-input').val();
                    }
                    if( actionName !== 'create' ){
                        if( issueRelatedCtags.length ){
                            let conf = confirm(
                                'カテゴリが変更されるため、このタグと脆弱性の紐付けが削除されます。よろしいですか?'
                            );
                            if( !conf ) {
                                $('#checking-window').remove();
                                button.prop('disabled', false);
                                return;
                            }
                        }
                    }
                    dynamicSummaryTable.executeProcess({
                        paramsData,
                        path: actionPath,
                        additionalAlertMessage: alertMessage
                    });
               }
            );
        })();
    });

    //一括削除
    function getBulkDeleteParam(){
        return {
            ids: JSON.parse($('#pages_checked').val()),
            paramsData: { ids: '' },
            idsParamsName: 'ids',
            path: $('#bulk-delete-ctags-path').val(),
            OneProcessSize: 1,
        }
    }
    $('#bulk-delete-ctags').on('click', function(){
        const allTagNames = JSON.parse($('#all-ctag-names').val());
        const checked = JSON.parse($('#pages_checked').val());
        const names = checked.map(id => {
            return `「${allTagNames[id]}」`;
        });
        (async () => {
            const roleOrCategoryError = await dynamicSummaryTable.checkRolesAndCatsBeforeAction({
                additionalAlertMessage: '削除に失敗しました。'
            });
            if( roleOrCategoryError ) return;
            $('body').append(create_loading_window());
            $.ajax({
                url: $('#sidfm_internal_api_check_related_rules_path').val(),
                type: 'POST',
                data: {'page_name': 'ctag', 'delete_ids': JSON.stringify(checked), 'action_idx': 'bulk_delete'},
            }).done(function(data){
                //console.log(data);
                if( data['is_unchangeable_ctag'] ){
                    $('#checking-window').find('.progress-wait-area-label').text('');
                    setTimeout(function(){
                        alert(
                            'ログインユーザのカテゴリ範囲外のルールの条件、操作もしくはルール自体が削除されるため、カスタムタグ削除することができません。\n' +
                            '※削除がどうしても必要な場合は、SIDfmVM管理者権限のユーザで、再度実行してください。'
                        );
                        $('#checking-window').remove();
                    }, 100);
                    return false;
                }
                if(Object.keys(data['rule_messages']).length > 0){
                    $('#submit-related-rules-item-form').attr({
                        'data-action': 'bulk_delete',
                    });
                    $('#checking-window').remove();
                    display_related_rules(data);
                }else{
                    let conf = confirm('以下のタグを削除します。よろしいですか?\n' + names.join(', '));
                    if( !conf ) {
                        $('#checking-window').remove();
                        return;
                    }
                    dynamicSummaryTable.executeProcessWithProgressBar(getBulkDeleteParam());
                }
            }).fail(function(){
                alert('エラーが発生しました。');
                $('#pages_form').submit();
            });
        })();
    });

    //ルール表示後の処理を実行
    $('#submit-related-rules-item-form').on('click', function(){
        const button = $(this);
        const actionName = button.attr('data-action');
        const params = button.attr('data-params');
        if( actionName === 'bulk_delete' ){
            dynamicSummaryTable.executeProcessWithProgressBar(getBulkDeleteParam());
        }else{
            dynamicSummaryTable.executeProcess({
                paramsData: JSON.parse(params),
                path: button.attr('data-path'),
                additionalAlertMessage: '更新に失敗しました。'
            });
        }
        return false;
    });

});
