require_dependency 'my_controller'

module SidfmPatches
  module MyControllerPatch
    extend ActiveSupport::Concern
   
    included do
      before_action :check_role, only: :account
      alias_method :password, :password_with_rescue_mail_error
      alias_method :page, :page_with_render_vm
    end
  
    def check_role
      if !User.current.admin then
        redirect_to home_path and return
      end
    end

    def password_with_rescue_mail_error
      @user = User.current
      unless @user.change_password_allowed?
        flash[:error] = l(:notice_can_t_change_password)
        redirect_to my_account_path
        return
      end
      if request.post?
        if !@user.check_password?(params[:password])
          flash.now[:error] = l(:notice_account_wrong_password)
        elsif params[:password] == params[:new_password]
          flash.now[:error] = l(:notice_new_password_must_be_different)
        else
          @user.password, @user.password_confirmation = params[:new_password], params[:new_password_confirmation]
          @user.must_change_passwd = false
          if @user.save 
            # The session token was destroyed by the password change, generate a new one
            session[:tk] = @user.generate_session_token
            begin
              Mailer.password_updated(@user)
            rescue  Net::SMTPFatalError => e
              flash[:password_change_mail_error] = true
            end
            flash[:notice] = l(:notice_account_password_updated)
            redirect_to my_account_path
          end
        end
      end
    end

    def page_with_render_vm
      redirect_to sidfm_index_path and return unless User.current.admin
      @user = User.current
      @groups = @user.pref.my_page_groups
      @blocks = @user.pref.my_page_layout
    end
  end
end

ActionDispatch::Reloader.to_prepare do
  unless MyController.included_modules.include?(SidfmPatches::MyControllerPatch)
    MyController.send(:include, SidfmPatches::MyControllerPatch)
  end
end

