#!/usr/bin/env ruby
# -*- coding: utf-8 -*-

class CreateSidfmdIssues < ActiveRecord::Migration

  def change
    # API経由で照合したSIDfmの情報を一時保持する。sidfmd_queueと共にexpireされる。
    create_table :sidfmd_queue_vuls do |t|
      t.references :sidfmd_queue, :null => false, :index => true
      t.integer :html_id, :null => false, :index => true
      t.string :title, :null => false
      t.integer :area_id, :null => false, :index => true
      t.integer :risk_id, :null => false, :index => true
    end
    create_table :sidfmd_queue_vul_cves do |t|
      t.references :sidfmd_queue_vul, :null => false, :index => true
      t.string :cve_id, :index => true
      t.string :c2_vector
      t.float :c2_base_score
      t.datetime :c2_create_datetime
      t.datetime :c2_update_datetime
      t.boolean :c2_exploitable
      t.string :c3_vector
      t.float :c3_base_score
      t.datetime :c3_create_datetime
      t.datetime :c3_update_datetime
      t.boolean :c3_exploitable
    end
    create_table :sidfmd_queue_vul_sris do |t|
      t.references :sidfmd_queue_vul, :null => false, :index => true
      t.integer :category_id, :null => false
      t.integer :risk_id, :null => false
      t.text :memo
    end
    create_table :sidfmd_queue_vul_affprods do |t|
      t.references :sidfmd_queue_vul, :null => false, :index => true
      t.integer :product_id, :null => false, :index => true
      t.string :name
      t.datetime :create_datetime
      t.datetime :update_datetime
    end
    create_table :sidfmd_queue_vul_affvers do |t|
      t.references :sidfmd_queue_vul, :null => false, :index => true
      t.integer :product_version_id, :null => false, :index => true
      t.integer :product_id, :null => false, :index => true
      t.string :fullname
      t.string :version
      t.datetime :create_datetime
      t.datetime :update_datetime
    end
    create_table :sidfmd_queue_vul_affpkgs do |t|
      t.references :sidfmd_queue_vul, :null => false, :index => true
      t.string :name
      t.string :version
      t.string :release
      t.string :arch
    end

    # Issueに紐づけてSIDfmの情報を保持する。Issueが削除されない限り、削除されることは
    # 無い。
    # 内容はほぼsidfmd_queueのコピー。
    create_table :sidfmd_issues do |t|
      t.references :issue, :null => false, :index => true
      t.references :sidfmd_host, :null => false, :index => true
      t.integer :html_id, :null => false, :index => true
      t.string :title, :null => false
      t.integer :area_id, :null => false, :index => true
      t.integer :risk_id, :null => false, :index => true
    end
    create_table :sidfmd_issue_cves do |t|
      t.references :sidfmd_issue, :null => false, :index => true
      t.string :cve_id, :index => true
      t.string :c2_vector
      t.float :c2_base_score
      t.datetime :c2_create_datetime
      t.datetime :c2_update_datetime
      t.boolean :c2_exploitable
      t.string :c3_vector
      t.float :c3_base_score
      t.datetime :c3_create_datetime
      t.datetime :c3_update_datetime
      t.boolean :c3_exploitable
    end
    create_table :sidfmd_issue_sris do |t|
      t.references :sidfmd_issue, :null => false, :index => true
      t.integer :category_id, :null => false
      t.integer :risk_id, :null => false
      t.text :memo
    end
    create_table :sidfmd_issue_affprods do |t|
      t.references :sidfmd_issue, :null => false, :index => true
      t.integer :product_id, :null => false, :index => true
      t.string :name
      t.datetime :create_datetime
      t.datetime :update_datetime
    end
    create_table :sidfmd_issue_affvers do |t|
      t.references :sidfmd_issue, :null => false, :index => true
      t.integer :product_version_id, :null => false, :index => true
      t.integer :product_id, :null => false, :index => true
      t.string :fullname
      t.string :version
      t.datetime :create_datetime
      t.datetime :update_datetime
    end
    create_table :sidfmd_issue_affpkgs do |t|
      t.references :sidfmd_issue, :null => false, :index => true
      t.string :name
      t.string :version
      t.string :release
      t.string :arch
    end

  end

end
